@@ -72,7 +72,8 @@ class CcrxCompiler(Compiler):
         self.warn_args = {'0': [],
                           '1': default_warn_args,
                           '2': default_warn_args + [],
-                          '3': default_warn_args + []}  # type: T.Dict[str, T.List[str]]
+                          '3': default_warn_args + [],
+                          'everything': default_warn_args + []}  # type: T.Dict[str, T.List[str]]
 
     def get_pic_args(self) -> T.List[str]:
         # PIC support is not enabled by default for CCRX,
@@ -107,7 +108,7 @@ class CcrxCompiler(Compiler):
         return ccrx_debug_args[is_debug]
 
     @classmethod
-    def _unix_args_to_native(cls, args: T.List[str], info: 'MachineInfo') -> T.List[str]:
+    def _unix_args_to_native(cls, args: T.List[str], info: MachineInfo) -> T.List[str]:
         result = []
         for i in args:
             if i.startswith('-D'):
