@@ -41,6 +41,7 @@ ti_buildtype_args = {
 }  # type: T.Dict[str, T.List[str]]
 
 ti_optimization_args = {
+    'plain': [],
     '0': ['-O0'],
     'g': ['-Ooff'],
     '1': ['-O1'],
@@ -70,7 +71,8 @@ class TICompiler(Compiler):
         self.warn_args = {'0': [],
                           '1': default_warn_args,
                           '2': default_warn_args + [],
-                          '3': default_warn_args + []}  # type: T.Dict[str, T.List[str]]
+                          '3': default_warn_args + [],
+                          'everything': default_warn_args + []}  # type: T.Dict[str, T.List[str]]
 
     def get_pic_args(self) -> T.List[str]:
         # PIC support is not enabled by default for TI compilers,
@@ -122,7 +124,7 @@ class TICompiler(Compiler):
         return ['-I=' + path]
 
     @classmethod
-    def _unix_args_to_native(cls, args: T.List[str], info: 'MachineInfo') -> T.List[str]:
+    def _unix_args_to_native(cls, args: T.List[str], info: MachineInfo) -> T.List[str]:
         result = []
         for i in args:
             if i.startswith('-D'):
