@@ -1,6 +1,7 @@
 # SPDX-License-Identifier: Apache-2.0
 # Copyright © 2021 The Meson Developers
 # Copyright © 2021 Intel Corporation
+from __future__ import annotations
 
 """Keyword Argument type annotations."""
 
@@ -11,7 +12,7 @@ from typing_extensions import TypedDict, Literal, Protocol
 from .. import build
 from .. import coredata
 from ..compilers import Compiler
-from ..mesonlib import MachineChoice, File, FileMode, FileOrString
+from ..mesonlib import MachineChoice, File, FileMode, FileOrString, OptionKey
 from ..modules.cmake import CMakeSubprojectOptions
 from ..programs import ExternalProgram
 
@@ -56,7 +57,7 @@ class FuncTest(FuncBenchmark):
 
     """Keyword Arguments for `test`
 
-    `test` only adds the `is_prallel` argument over benchmark, so inherintance
+    `test` only adds the `is_parallel` argument over benchmark, so inheritance
     is helpful here.
     """
 
@@ -202,7 +203,7 @@ class Project(TypedDict):
 
     version: T.Optional[FileOrString]
     meson_version: T.Optional[str]
-    default_options: T.List[str]
+    default_options: T.Dict[OptionKey, T.Union[str, int, bool, T.List[str]]]
     license: T.List[str]
     subproject_dir: str
 
@@ -211,7 +212,7 @@ class _FoundProto(Protocol):
 
     """Protocol for subdir arguments.
 
-    This allows us to define any objec that has a found(self) -> bool method
+    This allows us to define any object that has a found(self) -> bool method
     """
 
     def found(self) -> bool: ...
@@ -297,13 +298,13 @@ class ConfigureFile(TypedDict):
 
 class Subproject(ExtractRequired):
 
-    default_options: T.List[str]
+    default_options: T.Dict[OptionKey, T.Union[str, int, bool, T.List[str]]]
     version: T.List[str]
 
 
 class DoSubproject(ExtractRequired):
 
-    default_options: T.List[str]
+    default_options: T.Dict[OptionKey, T.Union[str, int, bool, T.List[str]]]
     version: T.List[str]
     cmake_options: T.List[str]
     options: T.Optional[CMakeSubprojectOptions]
