@@ -44,6 +44,10 @@ def add_arguments(parser: 'argparse.ArgumentParser') -> None:
                         help='Generate a native compilation file.')
     parser.add_argument('--system', default=None,
                         help='Define system for cross compilation.')
+    parser.add_argument('--subsystem', default=None,
+                        help='Define subsystem for cross compilation.')
+    parser.add_argument('--kernel', default=None,
+                        help='Define kernel for cross compilation.')
     parser.add_argument('--cpu', default=None,
                         help='Define cpu for cross compilation.')
     parser.add_argument('--cpu-family', default=None,
@@ -61,6 +65,8 @@ class MachineInfo:
         self.cmake: T.Dict[str, T.Union[str, T.List[str]]] = {}
 
         self.system: T.Optional[str] = None
+        self.subsystem: T.Optional[str] = None
+        self.kernel: T.Optional[str] = None
         self.cpu: T.Optional[str] = None
         self.cpu_family: T.Optional[str] = None
         self.endian: T.Optional[str] = None
@@ -131,13 +137,16 @@ def get_args_from_envvars(infos: MachineInfo) -> None:
     if objcpp_link_args:
         infos.link_args['objcpp'] = objcpp_link_args
 
-cpu_family_map = {
+deb_cpu_family_map = {
     'mips64el': 'mips64',
     'i686': 'x86',
+    'powerpc64le': 'ppc64',
 }
-cpu_map = {
+
+deb_cpu_map = {
     'armhf': 'arm7hlf',
-    'mips64el': 'mips64'
+    'mips64el': 'mips64',
+    'powerpc64le': 'ppc64',
 }
 
 def deb_detect_cmake(infos: MachineInfo, data: T.Dict[str, str]) -> None:
@@ -145,7 +154,10 @@ def deb_detect_cmake(infos: MachineInfo, data: T.Dict[str, str]) -> None:
     system_processor_map = {'arm': 'armv7l', 'mips64el': 'mips64', 'powerpc64le': 'ppc64le'}
 
     infos.cmake["CMAKE_C_COMPILER"] = infos.compilers['c']
-    infos.cmake["CMAKE_CXX_COMPILER"] = infos.compilers['cpp']
+    try:
+        infos.cmake["CMAKE_CXX_COMPILER"] = infos.compilers['cpp']
+    except KeyError:
+        pass
     infos.cmake["CMAKE_SYSTEM_NAME"] = system_name_map[data['DEB_HOST_ARCH_OS']]
     infos.cmake["CMAKE_SYSTEM_PROCESSOR"] = system_processor_map.get(data['DEB_HOST_GNU_CPU'],
                                                                      data['DEB_HOST_GNU_CPU'])
@@ -160,7 +172,7 @@ def deb_compiler_lookup(infos: MachineInfo, compilerstems: T.List[T.Tuple[str, s
             pass
 
 def detect_cross_debianlike(options: T.Any) -> MachineInfo:
-    if options.debarch is None:
+    if options.debarch == 'auto':
         cmd = ['dpkg-architecture']
     else:
         cmd = ['dpkg-architecture', '-a' + options.debarch]
@@ -175,10 +187,12 @@ def detect_cross_debianlike(options: T.Any) -> MachineInfo:
         data[k] = v
     host_arch = data['DEB_HOST_GNU_TYPE']
     host_os = data['DEB_HOST_ARCH_OS']
-    host_cpu_family = cpu_family_map.get(data['DEB_HOST_GNU_CPU'],
-                                         data['DEB_HOST_GNU_CPU'])
-    host_cpu = cpu_map.get(data['DEB_HOST_ARCH'],
-                           data['DEB_HOST_ARCH'])
+    host_subsystem = host_os
+    host_kernel = 'linux'
+    host_cpu_family = deb_cpu_family_map.get(data['DEB_HOST_GNU_CPU'],
+                                             data['DEB_HOST_GNU_CPU'])
+    host_cpu = deb_cpu_map.get(data['DEB_HOST_ARCH'],
+                               data['DEB_HOST_ARCH'])
     host_endian = data['DEB_HOST_ARCH_ENDIAN']
 
     compilerstems = [('c', 'gcc'),
@@ -199,7 +213,7 @@ def detect_cross_debianlike(options: T.Any) -> MachineInfo:
     except ValueError:
         pass
     try:
-        infos.binaries['pkgconfig'] = locate_path("%s-pkg-config" % host_arch)
+        infos.binaries['pkg-config'] = locate_path("%s-pkg-config" % host_arch)
     except ValueError:
         pass # pkg-config is optional
     try:
@@ -207,6 +221,8 @@ def detect_cross_debianlike(options: T.Any) -> MachineInfo:
     except ValueError:
         pass
     infos.system = host_os
+    infos.subsystem = host_subsystem
+    infos.kernel = host_kernel
     infos.cpu_family = host_cpu_family
     infos.cpu = host_cpu
     infos.endian = host_endian
@@ -254,11 +270,18 @@ def write_machine_file(infos: MachineInfo, ofilename: str, write_system_info: bo
             ofile.write(f"cpu_family = '{infos.cpu_family}'\n")
             ofile.write(f"endian = '{infos.endian}'\n")
             ofile.write(f"system = '{infos.system}'\n")
+            if infos.subsystem:
+                ofile.write(f"subsystem = '{infos.subsystem}'\n")
+            if infos.kernel:
+                ofile.write(f"kernel = '{infos.kernel}'\n")
+
     os.replace(tmpfilename, ofilename)
 
 def detect_language_args_from_envvars(langname: str, envvar_suffix: str = '') -> T.Tuple[T.List[str], T.List[str]]:
     ldflags = tuple(shlex.split(os.environ.get('LDFLAGS' + envvar_suffix, '')))
-    compile_args = shlex.split(os.environ.get(compilers.CFLAGS_MAPPING[langname] + envvar_suffix, ''))
+    compile_args = []
+    if langname in compilers.CFLAGS_MAPPING:
+        compile_args = shlex.split(os.environ.get(compilers.CFLAGS_MAPPING[langname] + envvar_suffix, ''))
     if langname in compilers.LANGUAGES_USING_CPPFLAGS:
         cppflags = tuple(shlex.split(os.environ.get('CPPFLAGS' + envvar_suffix, '')))
         lang_compile_args = list(cppflags) + compile_args
@@ -289,8 +312,16 @@ def detect_binaries_from_envvars(infos: MachineInfo, envvar_suffix: str = '') ->
         if binstr:
             infos.binaries[binname] = shlex.split(binstr)
 
+def detect_properties_from_envvars(infos: MachineInfo, envvar_suffix: str = '') -> None:
+    var = os.environ.get('PKG_CONFIG_LIBDIR' + envvar_suffix)
+    if var is not None:
+        infos.properties['pkg_config_libdir'] = var
+    var = os.environ.get('PKG_CONFIG_SYSROOT_DIR' + envvar_suffix)
+    if var is not None:
+        infos.properties['sys_root'] = var
+
 def detect_cross_system(infos: MachineInfo, options: T.Any) -> None:
-    for optname in ('system', 'cpu', 'cpu_family', 'endian'):
+    for optname in ('system', 'subsystem', 'kernel', 'cpu', 'cpu_family', 'endian'):
         v = getattr(options, optname)
         if not v:
             mlog.error(f'Cross property "{optname}" missing, set it with --{optname.replace("_", "-")}.')
@@ -305,6 +336,8 @@ def detect_cross_env(options: T.Any) -> MachineInfo:
         print('Detecting cross environment via environment variables.')
         infos = detect_compilers_from_envvars()
         detect_cross_system(infos, options)
+    detect_binaries_from_envvars(infos)
+    detect_properties_from_envvars(infos)
     return infos
 
 def add_compiler_if_missing(infos: MachineInfo, langname: str, exe_names: T.List[str]) -> None:
@@ -350,6 +383,7 @@ def detect_native_env(options: T.Any) -> MachineInfo:
     detect_missing_native_compilers(infos)
     detect_binaries_from_envvars(infos, esuffix)
     detect_missing_native_binaries(infos)
+    detect_properties_from_envvars(infos, esuffix)
     return infos
 
 def run(options: T.Any) -> None:
